//==========================================================================================
// Name         : sp4type.cpp
// Author       : Tom Hartmann
// Version      : 1.0
// Copyright    : The tool and the source code can freely be used and distributed for non-commercial
//		 	purposes under the condition that the manuscript "Models and Algorithms for Sorting 
//		 	Permutations with Tandem Duplication and Random Loss" is cited. 
// Description  : Tool solves the Sorting Problem and the Distance Problem under the rearrangement
//			model that considers inversions, transpositions, inverse transpositions, and TDRL
// Date		: 09.11.2018
//==========================================================================================

#include <iostream>
#include <getopt.h>
#include <string>
#include <stdlib.h>
#include <vector>
#include <set>

#include "helpers.hpp"
#include "genom.hpp"
#include "io.hpp"
#include "sb4type_lib.hpp"
#include "rearrangements.hpp"

using namespace std;


/**
 * get program parameters
 * @param[in] argc argument count
 * @param[in] argv arguments
 * @param[out] fname filename
 */
void getoptions(int argc, char *argv[], string &fname, bool &print_distance);

/**
 * print usage info and exit
 */
void usage();



int main(int argc, char *argv[]){
	string filename;			// input filename
	vector<genom> genomes;	 		// the genomes
	bool circular=false;			// consider genomes to be circular
	vector<string> names, 			// names of the genomes
		nmap;				// names of the elements
	bool print_distance=false;

	getoptions(argc, argv, filename, print_distance);

	if(filename==""){
		cerr << "No file name given ---> exiting!" << endl;
		usage();
	}

	read_genomes(filename, genomes, names, circular, nmap, false);	// read the gene order file
	if(genomes.size() == 0){
		cout << "No genomes in the file!" << endl;
		usage();
	}


	if(print_distance){//solve distance problem only and print results
			// build distance matrix
			std::vector< std::vector<int> > distances(genomes.size(), std::vector<int>(genomes.size(),0));

			//distances are calculated
			for(unsigned int i=0; i<genomes.size(); i++){
				for(unsigned int j=0; j<genomes.size(); j++){
					if(i!=j){
						if(genomes[i]==genomes[j]){
							cout << names[i] << "\t" << names[j] << "\t" << 0 << endl;
							continue;
						}
						set<int>::iterator it;
						// calculate the distance i -> j
						rrrmt* rhos;
						sort_by_4type(genomes[i],genomes[j], rhos);
						cout << names[i] << "\t" << names[j] << "\t" << rhos->length(ALTMIN) << endl;
					}
				}
			}
		}else{//solve sorting problem
			for(unsigned int i=0; i<genomes.size(); i++){
				for(unsigned int j=0; j< genomes.size(); j++){
					if(i!=j){
						if(genomes[i]==genomes[j]){
							cout << names[i] << " = " << names[j] << endl;
							cout << endl;
							continue;
						}
						set<int>::iterator it;
						// calculate the distance & scenario i -> j
						rrrmt* rhos;
						//sort_by_4type(genomes[i], sigmas, rhos);
						sort_by_4type(genomes[i], genomes[j], rhos);

//						cout << *rhos << endl;

						// print scenario
						cout << names[i] << " --> " << names[j] << ":" << endl;
						vector<rrrmt* > scen;
						rhos->getrrrmt(scen, ALTALL);
						genom temp_sg=genomes[i];

						cout << temp_sg << endl;
						for(int l=0; l<scen.size(); l++){
							cout << "\t" << *scen[l];
							scen[l]->apply(temp_sg);
							cout << temp_sg << endl;
						}
						cout << endl;

						// test
						if(temp_sg != genomes[j]){
							cout << "ERROR: Scenario does not transform ";
							cout << genomes[i] << " into " << genomes[j] << "---> exiting!" << endl;
							cout << *rhos << endl;
							exit(EXIT_FAILURE);
						}
					}//end i!=j
				}//end for j
			}//end for i
		}//end solve sorting problem

	// test the algorithm
//	test_sorting_algorithm(genomes[0]);
//	test_sorting_algorithm_pairwise(genomes[0]);
	return 0;
}

// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

void getoptions(int argc, char *argv[], string &fname, bool &print_distance){
	int c;
	while (1) {
		static struct option long_options[] = {
			{"file",    required_argument, 0, 'f'},
			{"distance", no_argument, 0, 'd'},
			{"help", no_argument, 0, 'h'},
			{0, 0, 0, 0}
		};
        int option_index = 0;			// getopt_long stores the option index here.
        c = getopt_long (argc, argv, "f:hd", long_options, &option_index);
        /* Detect the end of the options. */
        if (c == -1)
        	break;
        switch (c){
			case 0: // If this option set a flag, do nothing else now.
				if (long_options[option_index].flag != 0)
					break;
				cout << "option "<< long_options[option_index].name;
				if (optarg)
					cout << " with arg " << optarg;
				cout << endl;
				break;
			case 'f':
				fname = optarg;
				break;
			case 'd':
				print_distance=true;
				break;
			case 'h':
				usage();
				exit(EXIT_SUCCESS);
			case '?':
				usage();
				exit(EXIT_SUCCESS);
				break; /* getopt_long already printed an error message. */
			default:
				usage();
        }
	}
}

// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

void usage(){
	cerr << "The tool SP4TYPE solves the sorting problem and the distance problem for signed linear permutations ";
	cerr <<	"under the rearrangement model that considers inversions, transpositions, TDRLs, and inverse transpositions. ";
	cerr << "The resulting scenarios of rearrangements deviate from a parsimonious scenario in at most two rearrangements. ";
	cerr << "SP4TYPE has been described in the PhD Thesis \"Models and Algorithms for Sorting Permutations with Tandem Duplication and Random Loss\" ";
	cerr << "by T. Hartmann (2018)" << endl;
	cerr << " --file	-f	<fasta file>, contains (multiple gene orders)" << endl;
	cerr << " --distance	-d	only distance problem is considered" << endl;
	cerr << " --help	-h	prints this message" << endl;
	cerr << "required argument: f" << endl;
	exit(1);
}
